@@ -2,14 +2,18 @@ use crate::{Alpm, Error, LoadedPackage, Package};
 
 use alpm_sys::*;
 
-pub unsafe trait IntoPkgAdd {
+use std::fmt;
+
+pub unsafe trait IntoPkgAdd: fmt::Debug {
+    #[doc(hidden)]
     unsafe fn as_alpm_pkg_t(&self) -> *mut alpm_pkg_t;
+    #[doc(hidden)]
     unsafe fn added(self);
 }
 
 unsafe impl<'a> IntoPkgAdd for Package<'a> {
     unsafe fn as_alpm_pkg_t(&self) -> *mut alpm_pkg_t {
-        self.pkg
+        self.pkg.pkg
     }
     unsafe fn added(self) {}
 }
@@ -23,7 +27,7 @@ unsafe impl<'a> IntoPkgAdd for LoadedPackage<'a> {
 }
 
 impl Alpm {
-    pub fn trans_add_pkg<P: IntoPkgAdd>(&self, pkg: P) -> std::result::Result<(), AddError<P>> {
+    pub fn trans_add_pkg<P: IntoPkgAdd>(&self, pkg: P) -> Result<(), AddError<P>> {
         let ret = unsafe { alpm_add_pkg(self.handle, pkg.as_alpm_pkg_t()) };
         let ok = self.check_ret(ret);
         match ok {
@@ -42,8 +46,16 @@ pub struct AddError<P> {
     pub pkg: P,
 }
 
-impl<P> Into<Error> for AddError<P> {
-    fn into(self) -> Error {
-        self.err
+impl<P> fmt::Display for AddError<P> {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        fmt::Display::fmt(&self.err, f)
+    }
+}
+
+impl<P: IntoPkgAdd> std::error::Error for AddError<P> {}
+
+impl<P> From<AddError<P>> for Error {
+    fn from(err: AddError<P>) -> Error {
+        err.err
     }
 }
