@@ -14,17 +14,17 @@ use std::{fmt, ptr};
 use alpm_sys::*;
 
 pub trait AsPkg {
-    fn as_pkg<'a>(&'a self) -> Pkg<'a>;
+    fn as_pkg(&self) -> Pkg;
 }
 
 impl<'a> AsPkg for Package<'a> {
-    fn as_pkg<'b>(&'b self) -> Pkg<'b> {
+    fn as_pkg(&self) -> Pkg {
         self.pkg
     }
 }
 
 impl<'a> AsPkg for Pkg<'a> {
-    fn as_pkg<'b>(&'b self) -> Pkg<'b> {
+    fn as_pkg(&self) -> Pkg {
         *self
     }
 }
@@ -91,7 +91,7 @@ impl<'a> Pkg<'a> {
 
     pub fn filename(&self) -> &'a str {
         let name = unsafe { alpm_pkg_get_filename(self.pkg) };
-        unsafe { from_cstr(name) }
+        unsafe { from_cstr_optional2(name) }
     }
 
     pub fn base(&self) -> Option<&'a str> {
@@ -322,6 +322,7 @@ mod tests {
         }
 
         assert!(files.contains("etc/").unwrap().is_some());
+        assert_eq!(pkg.filename(), "");
     }
 
     #[test]
